var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import PropTypes from 'prop-types';
import UniversalRussianRouter from './UniversalRussianRouter';
import { utils } from 'russian-router';

var pathRegExp = /\/$/;
var Protocol = utils.getPartConstructor('protocol');
var Domain = utils.getPartConstructor('domain');
var Port = utils.getPartConstructor('port');

var isLeftButtonClick = function isLeftButtonClick(event) {
    return event.button === 0;
};

var isMetaKeyPressed = function isMetaKeyPressed(event) {
    return event.metaKey || event.ctrlKey || event.altKey || event.shiftKey;
};

var queryToLowerCase = function queryToLowerCase(queryObject) {
    var newQueryObject = {};
    for (var q in queryObject) {
        newQueryObject[q.toLowerCase()] = queryObject[q].toLowerCase();
    }
    return newQueryObject;
};

var compareStrings = function compareStrings(linkString, routerString, routeOptions) {
    if (!routeOptions.caseSensitive) {
        linkString = linkString.toLowerCase();
        routerString = routerString.toLowerCase();
    }
    return linkString === routerString;
};

var compareProtocols = function compareProtocols(linkProtocol, routerProtocol) {
    if (!linkProtocol) {
        return true;
    }
    return compareStrings(linkProtocol, routerProtocol, { caseSensitive: false });
};

var compareDomains = function compareDomains(linkDomain, routerDomain) {
    if (!linkDomain) {
        return true;
    }
    return compareStrings(linkDomain, routerDomain, { caseSensitive: false });
};

var comparePorts = function comparePorts(linkPort, routerPort) {
    return compareStrings(linkPort, routerPort, { caseSensitive: false });
};

var comparePaths = function comparePaths(linkPath, routerPath, routeOptions) {
    if (!routeOptions.trailingSlashSensitive) {
        linkPath = linkPath.replace(pathRegExp, '');
        routerPath = routerPath.replace(pathRegExp, '');
    }
    return compareStrings(linkPath, routerPath, routeOptions);
};

var compareQueries = function compareQueries(linkQuery, routerQuery, routeOptions, isStrongMode) {
    if (!routeOptions.caseSensitive) {
        linkQuery = queryToLowerCase(linkQuery);
        routerQuery = queryToLowerCase(routerQuery);
    }
    var linkQueryKeys = Object.keys(linkQuery);
    var routerQueryKeys = Object.keys(routerQuery);
    if (routerQueryKeys.length < linkQueryKeys.length) {
        return false;
    }
    if (isStrongMode && linkQueryKeys.sort().join() !== routerQueryKeys.sort().join()) {
        return false;
    }

    for (var l in linkQuery) {
        if (routerQuery[l] === undefined) {
            return false;
        }
        if (linkQuery[l] !== routerQuery[l]) {
            return false;
        }
    }
    return true;
};

var compareHashes = function compareHashes(linkHash, routerHash, routeOptions, isStrongMode) {
    if (!linkHash && !isStrongMode) {
        return true;
    }
    return compareStrings(linkHash, routerHash, routeOptions);
};

var Link = function (_React$Component) {
    _inherits(Link, _React$Component);

    function Link() {
        _classCallCheck(this, Link);

        return _possibleConstructorReturn(this, (Link.__proto__ || Object.getPrototypeOf(Link)).apply(this, arguments));
    }

    _createClass(Link, [{
        key: 'render',
        value: function render() {
            var href = this._generateUri();
            var props = Object.assign({}, this.props);
            delete props.name;
            delete props.params;
            delete props.action;
            delete props.actionIfMatched;

            return React.createElement(
                'a',
                _extends({}, props, {
                    href: href,
                    className: this._getClassName(),
                    onClick: this._onClick.bind(this, href) }),
                this.props.children
            );
        }
    }, {
        key: 'shouldComponentUpdate',
        value: function shouldComponentUpdate(nextProps) {
            var router = this.context.router;

            var navigationKey = router.getNavigationKey();
            if (navigationKey !== this._navigationKey) {
                this._navigationKey = navigationKey;
                return true;
            }

            var props = this.props;

            var keys = Object.keys(props);
            var nextKeys = Object.keys(nextProps);
            if (keys.length !== nextKeys.length) {
                return true;
            }
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = nextKeys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var nextKey = _step.value;

                    if (nextProps[nextKey] !== props[nextKey]) {
                        return true;
                    }
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            return false;
        }
    }, {
        key: 'componentWillMount',
        value: function componentWillMount() {
            var router = this.context.router;

            this._navigationKey = router.getNavigationKey();
        }
    }, {
        key: '_getClassName',
        value: function _getClassName() {
            var className = this.props.className;

            return (className || '') + '';
        }
    }, {
        key: '_generateUri',
        value: function _generateUri() {
            var _props = this.props,
                href = _props.href,
                name = _props.name,
                params = _props.params;

            if (!name) {
                return href;
            }

            var router = this.context.router;

            return router.generateUri(name, params);
        }
    }, {
        key: '_isAnotherResource',
        value: function _isAnotherResource() {
            var router = this.context.router;

            var linkUri = this._generateUri().trim();
            var splittedLinkUri = utils.splitUri(linkUri, utils.getRegExp('uri'));

            var protocol = splittedLinkUri.protocol;
            var domain = splittedLinkUri.domain;
            var port = utils.getPortByParsedUri({
                protocol: new Protocol(protocol),
                domain: new Domain(domain),
                port: new Port(splittedLinkUri.port)
            }, { router: router }).toString();

            var defaultProtocol = router.getDefaultPart('protocol');
            var defaultDomain = router.getDefaultPart('domain');
            var defaultPort = router.getDefaultPart('port');

            return false || !compareProtocols(protocol || '', defaultProtocol.toString()) || !compareDomains(domain || '', defaultDomain.toString()) || !comparePorts(port || '', defaultPort.toString());
        }
    }, {
        key: '_isMatched',
        value: function _isMatched() {
            var isStrongMode = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
            var router = this.context.router;

            var linkParsedRoutes = {};
            var linkHref = this._generateUri();

            var routerParsedRoutes = router.getParsedRoutes();
            var routerMatchObjects = router.getMatchObjects();
            var routerMatchObjectsByName = {};
            routerMatchObjects.forEach(function (routerMatchObject) {
                var routeName = routerMatchObject.name;
                routerMatchObjectsByName[routeName] = routerMatchObject;
                var routerParsedRoute = routerParsedRoutes[routeName];
                linkParsedRoutes[routeName] = routerParsedRoute;
            });

            var linkMatchObjects = router.matchUri(linkHref, linkParsedRoutes);
            var linkMatchObjectsByName = {};
            linkMatchObjects.forEach(function (linkMatchObject) {
                var routeName = linkMatchObject.name;
                linkMatchObjectsByName[routeName] = linkMatchObject;
            });

            for (var routeName in linkMatchObjectsByName) {
                var routeOptions = router.getParsedRoutes()[routeName].getParsedOptions();
                var linkMatchObject = linkMatchObjectsByName[routeName];
                var routerMatchObject = routerMatchObjectsByName[routeName];
                if (compareProtocols(linkMatchObject.protocol, routerMatchObject.protocol, routeOptions) && compareDomains(linkMatchObject.domain, routerMatchObject.domain, routeOptions) && comparePorts(linkMatchObject.port, routerMatchObject.port, routeOptions) && comparePaths(linkMatchObject.path, routerMatchObject.path, routeOptions) && compareQueries(linkMatchObject.query, routerMatchObject.query, routeOptions, isStrongMode) && compareHashes(linkMatchObject.hash, routerMatchObject.hash, routeOptions, isStrongMode)) {
                    return true;
                }
            }

            return false;
        }
    }, {
        key: '_onClick',
        value: function _onClick(uri, event) {
            var onClick = this.props.onClick;

            if (onClick) {
                onClick(event);
            }
            if (event.defaultPrevented) {
                return;
            }
            if (this.props.target !== '_self') {
                return;
            }
            if (!isLeftButtonClick(event)) {
                return;
            }
            if (isMetaKeyPressed(event)) {
                return;
            }
            if (this._isAnotherResource()) {
                return;
            }

            event.preventDefault();
            if (!this.props.actionIfMatched && this._isMatched(true)) {
                return;
            }
            var router = this.context.router;
            var action = this.props.action;

            if (action === 'replace') {
                router.replaceUri(uri);
            } else {
                router.pushUri(uri);
            }
        }
    }]);

    return Link;
}(React.Component);

export default Link;


Link.contextTypes = {
    router: PropTypes.instanceOf(UniversalRussianRouter).isRequired
};

Link.propTypes = {
    href: PropTypes.string,
    name: PropTypes.string,
    params: PropTypes.object,
    action: PropTypes.oneOf(['replace', 'push']),
    target: PropTypes.string,
    actionIfMatched: PropTypes.bool,
    className: PropTypes.string,
    onClick: PropTypes.func
};

Link.defaultProps = {
    action: 'push',
    target: '_self',
    actionIfMatched: false
};